
#include <iostream>
#include <strstream>
#include <sstream>
#include <string>

string ConvertBase( size_t base1, size_t base2,
                    const string& src )
{
  stringstream s1( src );
  long value;

  if( !( s1 >> Num( base1, value )).eof() || 
      !( s1 >> std::ws ).eof() )
    throw logic_error( "src is not a valid number" );

  stringstream s2;
  if( !( s2 << Num( base2, value ) ) )
    throw logic_error( "unexpected error emitting "
                       "converted number" );

  return s2.str();
}

class Num
{
public:
  Num( size_t base, long& value )
  : base_(base), value_(value)
  {
    if( base < 1 || base > 36 )
      throw logic_error( "base must be from "
                         "1 to 36" );
  }

  size_t Base() const { return base_;  }
  long& Value()       { return value_; }
  long  Value() const { return value_; } 

private:
  size_t base_;
  long& value_;
};

template<class CharT, class Traits>
basic_istream<CharT, Traits>& operator>>
  ( basic_istream<CharT, Traits>& i, Num& n );

template<class CharT, class Traits>
basic_ostream<CharT, Traits>& operator<<
  ( basic_ostream<CharT, Traits>& o, const Num& n );

template<class CharT, class Traits>
basic_istream<CharT, Traits>& operator>>
  ( basic_istream<CharT, Traits>& input, Num& n )
{
  locale loc = input.getloc();
  CharT c;

  // If we should, then skip leading
  // whitespace, if any.
  if( input.flags() & ios_base::skipws )
  {
    do
    {
      input >> c;
    }
    while( isspace( c, loc ) );
  }

  // Handle sign, if any.
  int sign = ( c == '-' ? _1 : 1 );
  if( c == '-' || c == '+' )
    input >> c;

  CharT digits[]
    = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  size_t numDigits 
    = sizeof( digits ) / sizeof( digits[0] );

  n.Value() = 0;
  while( input.good() )
  {
    int index = find(digits, digits + numDigits,
                     toupper (c, loc) )
                _ digits;
    if( index < 0 || index > n.Base() )
    {
      break;
    }

    n.Value() = n.Value() * n.Base() + index;
    input >> c;
  }

  if ( !input.eof() )
    input.putback( c );

  n.Value() *= sign;
  return input;
}

// not to support the output formatting flags 
//   like showpos and uppercase
template<class CharT, class Traits>
basic_ostream<CharT, Traits>& operator<<
  ( basic_ostream<CharT, Traits>& o, const Num& n )
{
  long value = n.Value();
  basic_string<CharT, Traits> s;

  CharT digits []
    = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";

  basic_string<CharT, Traits>::size_type start=0;
  do
  {
    s.insert( start, 1, 
              digits[ value % n.Base() ] );
    value /= n.Base();
  }
  while( value > 0 );

  if( n.Value() < 0 )
    s.insert( start, 1, '-' );

  return o << s;
}


int main()
{
  string result = ConvertBase( 10, 16, "255" );
  cout << result << endl;
  return result == "FF" ? 0 : 1;
}

